const engineUrl = 'engine-rest/';
let camundaEngineUrl = '';
let connectionSuccess = false;
let $camundaApiRoot = $('#camunda-api-root');
let $textCamundaUrl = $(`
  <input class="modal-element" type="text" size="60">
`);
$textCamundaUrl.val('localhost');
let $buttonConnectCamundaUrl = $(`
  <button class="clickable modal-element" size="60" id="camunda-connect" 
  onclick="connectCamunda()">Connect</button>
`);
let $selectCamundaDecisions = $(
    `<select  class="clickable modal-element" name="decisions" 
      id="select-camunda-decisions" 
      onchange="loadCamundaDecisionXml(this)">
    `);
let $buttonDeployDmn = $(`
  <button class="clickable modal-element" size="60" id="camunda-connect" 
  onclick="deployDmn()">Deploy DMN</button>
`);
let $camundaApiContainer = $(`<div id="camunda-api-container">`);

// register Button action
$(document).ready(function () {
  clearCamundaApiRoot();
});

function clearCamundaApiRoot() {
  $camundaApiRoot.empty();
  $camundaApiRoot.append($(`<h2>Camunda DMN Deployment</h2>`));
  $camundaApiRoot.append($camundaApiContainer);
  loadCamundaSessionParams();
}

function refreshParams() {
  $textCamundaUrl.val(camundaEngineUrl);
  $camundaApiContainer.empty();
  if (connectionSuccess) {
    $camundaApiContainer.append($selectCamundaDecisions);
    $camundaApiContainer.append($buttonDeployDmn);
  } else {
    $camundaApiContainer.append($textCamundaUrl);
    $camundaApiContainer.append($buttonConnectCamundaUrl);
  }
}

function connectCamunda() {
  camundaEngineUrl = $textCamundaUrl.val();
  if (!camundaEngineUrl.endsWith('/')) {
    camundaEngineUrl += '/';
  }
  if (!camundaEngineUrl.startsWith('http://')) {
    camundaEngineUrl = 'http://' + camundaEngineUrl;
  }
  $textCamundaUrl.val(camundaEngineUrl);
  requestOptionsCamundaDecisions();
}

function deployDmn() {
  let dmnXml = exportDiagram();
  if (dmnXml) {
    if (confirm("Do you really want to deploy this dmn?")) {
      var fileStringArray = [dmnXml];

      var fileName = "file.dmn";
      var blobAttrs = {type: "xml"};
      var file = new File(fileStringArray, fileName, blobAttrs);

      var formData = new FormData();
      formData.append("deployment-name", "test");
      formData.append("enable-duplicate-filtering", "true");
      formData.append("deployment-source", "process application");
      formData.append("data", file, file.name);

      $.ajax({
        url: camundaEngineUrl + engineUrl + "deployment/create",
        type: "POST",
        data: formData,
        enctype: 'multipart/form-data',
        async: true,
        crossDomain: true,
        processData: false,
        contentType: false,
        cache: false,
        timeout: 0,
        success: function (data) {
          $.modal.close();
          requestOptionsCamundaDecisions();
        },
        error: function (e) {
          alert("There was a error while deploying the dmn.");
        }
      });
    }
  }
}

function saveCamundaUrl() {
  $.ajax({
    timeout: 1000,
    url: "/dmn/setCamundaUrl",
    type: 'POST',
    contentType: 'text/plain',
    data: $textCamundaUrl.val(),
    error: function (err) {
      // nothing
      console.log(err);
    }
  });
}

function loadCamundaSessionParams() {
  $.ajax({
    timeout: 1000,
    url: "/dmn",
    type: 'GET',
    contentType: 'application/json',
    error: function (err) {
      camundaEngineUrl = '';
      connectionSuccess = false;

      refreshParams();
    },
    success: function (data) {
      if (data !== undefined && data.camundaUrl !== undefined) {
        camundaEngineUrl = data.camundaUrl;
      } else {
        camundaEngineUrl = '';
      }
      refreshParams();
    }
  });
}

function requestOptionsCamundaDecisions() {
  $selectCamundaDecisions.empty();
  $.ajax({
    timeout: 1000,
    url: camundaEngineUrl + engineUrl
        + 'decision-definition?latestVersion=true',
    type: 'GET',
    contentType: 'application/json',
    error: function (err) {
      connectionSuccess = false;
      alert(
          "Error: No connection to the camunda engine established. Check URL or CORS options.");
      refreshParams();
    },
    success: function (data) {
      connectionSuccess = true;
      renderDecisionsOptions(data);
      saveCamundaUrl();
      refreshParams();
    }
  });
}

function renderDecisionsOptions(decisions) {
  $selectCamundaDecisions.empty();
  $selectCamundaDecisions.append($(`
    <option value="null" selected>Select a definition</option>
  `));
  for (let i = 0; i < decisions.length; i++) {
    let decision = decisions[i];
    $selectCamundaDecisions.append($(`
      <option value="${decision.key}">${decision.name} (Version: ${decision.version})</option>
    `));
  }
}

function loadCamundaDecisionXml(decisionSelect) {
  if (decisionSelect.value !== 'null') {
    if (confirm(
        'Do you really want to load the dmn "'
        + $("#select-camunda-decisions option:selected").text()
        + '"?')) {
      $.ajax({
        timeout: 1000,
        url: camundaEngineUrl + engineUrl + 'decision-definition/key/'
            + decisionSelect.value
            + '/xml/',
        type: 'GET',
        error: function (err) {
          console.log("error", err);
          connectionSuccess = false;
          refreshParams();
          alert(err);
        },
        success: function (data) {
          openDiagram(data.dmnXml);
          refreshParams();
          $.modal.close();
        }
      });
    }
  }
}
